@@ -443,50 +443,47 @@ button_load_pixbuf (const char  *file,
 #undef PREFERRED_SIZE
 }
 
-static void
-draw_arrow (GdkPoint         *points,
-	    PanelOrientation  orientation,
-	    int               width,
-	    int               height)
+static GtkArrowType
+calc_arrow (PanelOrientation  orientation,
+	    int               button_width,
+	    int               button_height,
+	    int              *x,
+	    int              *y,
+	    int              *width,
+	    int              *height)
 {
+	GtkArrowType retval = GTK_ARROW_UP;
 	double scale;
 
-	scale = (orientation & PANEL_HORIZONTAL_MASK ? height : width) / 48.0;
+	scale = (orientation & PANEL_HORIZONTAL_MASK ? button_height : button_width) / 48.0;
+
+	*width  = 12 * scale;
+	*height = 12 * scale;
 
 	switch (orientation) {
 	case PANEL_ORIENTATION_TOP:
-		points [0].x = 4 * scale;
-		points [0].y = height - 10 * scale;
-		points [1].x = 12 * scale;
-		points [1].y = height - 10 * scale;
-		points [2].x = 8 * scale;
-		points [2].y = height - 3 * scale;
+		*x     = scale * 3;
+		*y     = scale * (48 - 13);
+		retval = GTK_ARROW_DOWN;
 		break;
 	case PANEL_ORIENTATION_BOTTOM:
-		points [0].x = width - 12 * scale;
-		points [0].y = 10 * scale;
-		points [1].x = width - 4 * scale;
-		points [1].y = 10 * scale;
-		points [2].x = width - 8 * scale;
-		points [2].y = 3 * scale;
+		*x     = scale * (48 - 13);
+		*y     = scale * 1;
+		retval = GTK_ARROW_UP;
 		break;
 	case PANEL_ORIENTATION_LEFT:
-		points [0].x = width  - 10 * scale;
-		points [0].y = height - 12 * scale;
-		points [1].x = width  - 10 * scale;
-		points [1].y = height - 4 * scale;
-		points [2].x = width  - 3 * scale;
-		points [2].y = height - 8 * scale;
+		*x     = scale * (48 - 13);
+		*y     = scale * 3;
+		retval = GTK_ARROW_RIGHT;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
-		points [0].x = 10 * scale;
-		points [0].y = 4  * scale;
-		points [1].x = 10 * scale;
-		points [1].y = 12 * scale;
-		points [2].x = 3  * scale;
-		points [2].y = 8  * scale;
+		*x     = scale * 1;
+		*y     = scale * 3;
+		retval = GTK_ARROW_LEFT;
 		break;
 	}
+
+	return retval;
 }
 
 static gboolean
@@ -554,15 +551,32 @@ button_widget_expose (GtkWidget         *widget,
 	g_object_unref (pb);
 	
 	if (button_widget->arrow) {
-		int i;
-		GdkPoint points[3];
-		draw_arrow (points, button_widget->orientation, widget->allocation.width, widget->allocation.height);
-		for (i = 0; i < 3; i++) {
-			points[i].x += off + widget->allocation.x;
-			points[i].y += off + widget->allocation.y;
-		}
-		gdk_draw_polygon (widget->window, widget->style->white_gc, TRUE, points, 3);
-		gdk_draw_polygon (widget->window, widget->style->black_gc, FALSE, points, 3);
+		GtkArrowType arrow_type;
+		int          x, y, width, height;
+
+		x = y = width = height = -1;
+
+		arrow_type = calc_arrow (button_widget->orientation,
+					 widget->allocation.width,
+					 widget->allocation.height,
+					 &x,
+					 &y,
+					 &width,
+					 &height);
+
+		gtk_paint_arrow (widget->style,
+				 widget->window,
+				 GTK_STATE_NORMAL,
+				 GTK_SHADOW_NONE,
+				 NULL,
+				 widget,
+				 NULL,
+				 arrow_type,
+				 TRUE,
+				 widget->allocation.x + x,
+				 widget->allocation.y + y,
+				 width,
+				 height);
 	}
 
 	if (button_widget->dnd_highlight) {
@@ -654,17 +668,13 @@ button_widget_activate (GtkButton *button)
 static gboolean
 button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 {
- 	ButtonWidget *button;
-
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
-	button = BUTTON_WIDGET (widget);
+	if (event->button == 1 && BUTTON_WIDGET (widget)->activatable)
+		return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
 
-	if (!button->activatable)
-		return TRUE;
-
-	return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+	return FALSE;
 }
 
 static gboolean
@@ -751,8 +761,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_ACTIVATABLE,
 			g_param_spec_boolean ("activatable",
-					      _("Activatable"),
-					      _("Whether the button is activatable"),
+					      "Activatable",
+					      "Whether the button is activatable",
 					      TRUE,
 					      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
@@ -760,8 +770,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_HAS_ARROW,
 			g_param_spec_boolean ("has-arrow",
-					      _("Has Arrow"),
-					      _("Whether or not to draw an arrow indicator"),
+					      "Has Arrow",
+					      "Whether or not to draw an arrow indicator",
 					      FALSE,
 					      G_PARAM_READWRITE));
 
@@ -769,8 +779,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_DND_HIGHLIGHT,
 			g_param_spec_boolean ("dnd-highlight",
-					      _("Drag and drop Highlight"),
-					      _("Whether or not to highlight the icon during drag and drop"),
+					      "Drag and drop Highlight",
+					      "Whether or not to highlight the icon during drag and drop",
 					      FALSE,
 					      G_PARAM_READWRITE));
 
@@ -778,8 +788,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_ORIENTATION,
 			g_param_spec_enum ("orientation",
-					   _("Orientation"),
-					   _("The ButtonWidget orientation"),
+					   "Orientation",
+					   "The ButtonWidget orientation",
 					   PANEL_TYPE_ORIENTATION,
 					   PANEL_ORIENTATION_TOP,
 					   G_PARAM_READWRITE));
@@ -788,8 +798,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_ICON_NAME,
 			g_param_spec_string ("icon-name",
-					     _("Icon Name"),
-					     _("The desired icon for the ButtonWidget"),
+					     "Icon Name",
+					     "The desired icon for the ButtonWidget",
 					     NULL,
 					     G_PARAM_READWRITE));
 
@@ -797,8 +807,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			gobject_class,
 			PROP_STOCK_ID,
 			g_param_spec_string ("stock-id",
-					     _("Stock Icon ID"),
-					     _("The desired stock icon for the ButtonWidget"),
+					     "Stock Icon ID",
+					     "The desired stock icon for the ButtonWidget",
 					     NULL,
 					     G_PARAM_READWRITE));
 }
